/* fnvrand: a fast pseudo-random number generator
   v2004-07-12 / contact adam@gimp.org */
/*
Copyright (C) 2002-2004 Adam D. Moss (the "Author").  All Rights Reserved.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is fur-
nished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FIT-
NESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CON-
NECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the Author of the
Software shall not be used in advertising or otherwise to promote the sale,
use or other dealings in this Software without prior written authorization
from the Author.
*/

#define FNV1_32_INIT ((unsigned long)0x811c9dc5)
#define FNV_32_PRIME ((unsigned long)0x01000193)

#include "fnvrand.h"

static unsigned long seed = 0;
static unsigned long mixer = FNV1_32_INIT;


void
fnvsrand(unsigned long newseed)
{
  seed = newseed;
  mixer = FNV1_32_INIT;
}


unsigned long
fnvrand32(void)
{
  /* gcc generates better, faster code if >>24 is first.  Curious. */
  mixer ^= (unsigned char)(seed >> 24);
  mixer *= FNV_32_PRIME;
  mixer ^= (unsigned char)(seed >> 0);
  mixer *= FNV_32_PRIME;
  mixer ^= (unsigned char)(seed >> 8);
  mixer *= FNV_32_PRIME;
  mixer ^= (unsigned char)(seed >> 16);
  mixer *= FNV_32_PRIME;
  ++seed;
  return mixer;
}


unsigned short
fnvrand16(void)
{
  /* fold our 32 bits of entropy down to 16 bits */
  const unsigned long rnd = fnvrand32();
  return rnd ^ (rnd>>16);
}


unsigned char
fnvrand8(void)
{
  /* fold our 16 bits of entropy down to 8 bits */
  const unsigned short rnd = fnvrand16();
  return rnd ^ (rnd>>8);
}


unsigned char
fnvrand1(void)
{
  /* could do better, but this really isn't bad and it saves doing
     three more entropy-folding steps. */
  return 1 & fnvrand8();
}


unsigned char
fnvrand1best(void)
{
  /* this is the proper way to get 1 bit of randomness -- it's slower than
     fnvrand1() but the bit returned is a little more 'random'. */
  unsigned char rnd = fnvrand8();
  rnd = rnd ^ (rnd >> 4);
  rnd = rnd ^ (rnd >> 2);
  rnd = rnd ^ (rnd >> 1);
  return 1 & rnd;
}
